<?php
/*
Plugin Name: WP Super Cache Cleaner
Plugin URI: http://w-shadow.com/
Description: Cleans expired items from WP Super Cache. Can be set to run either daily or hourly. 
Version: 1.0
Author: Janis Elsts
Author URI: http://w-shadow.com/blog/
*/

/*
Created by Janis Elsts (email : whiteshadow@w-shadow.com) 
It's GPL.
*/

class CacheCleaner {
	var $options;
	var $option_name='ws_cache_cleaner';
	var $defaults;
	
	
	function CacheCleaner(){
		$this->load_options();
		
		add_action('activate_'.plugin_basename(__FILE__), array(&$this,'activation'));
		add_action('deactivate_'.plugin_basename(__FILE__), array(&$this,'deactivation'));
		add_action('clear_expired_cache', array(&$this,'clean_cache'));
		add_action('admin_menu', array(&$this, 'admin_menu'));
		
		//add_action('admin_footer', array(&$this,'admin_footer')); //debug
	}
	
	function activation(){
		$this->save_options(); //write the defaults to DB if this is the first activation
		//Re-schedule the cache cleaner hook.
		wp_clear_scheduled_hook('clear_expired_cache');
		wp_schedule_event(time(), $this->options['period'], 'clear_expired_cache');
	}
	
	function deactivation(){
		//Remove the hook when deactivated
		wp_clear_scheduled_hook('clear_expired_cache');
	}
	
	function admin_footer(){
		echo '<pre>', print_r($this->options, true);
		$timestamp = wp_next_scheduled('clear_expired_cache');
		echo "Next check : ",date('r', $timestamp);		 
		echo '</pre>';
	}
	
	function load_options(){
		$this->defaults = array(
			'period' => 'daily', //How often expired files should be deleted.
			'last_check' => 0, //When was ws_clean_expired_cache() last called. Used for debugging.
			'last_run' => 0, //When was the last time expired files were deleted (by this plugin).
		);
		$this->options = get_option($this->option_name);
		if(!is_array($this->options)){
			$this->options = $this->defaults;
			return false;
		} else {
			$this->options = array_merge($this->defaults, $this->options);
			return true;
		}
	}
	
	function save_options(){
		update_option($this->option_name, $this->options);
	}
	
	function clean_cache(){
		global $file_prefix;

		$this->options['last_check'] = time();
		if ( function_exists('wp_cache_clean_expired') ) {
			$delta = time() - $this->options['last_run'];
			
			if ( ( ('hourly' == $this->options['period']) && ($delta >= 3600) ) ||
				 ( ('daily' == $this->options['period']) && ($delta >= 86400) ) )
			{
				wp_cache_clean_expired($file_prefix);
				$this->options['last_run'] = time();
			} 
		}	
		
		$this->save_options();
		
		return true;
	}
	
	function admin_menu(){
		add_options_page('WP Super Cache Cleaner Settings', 'Cache Cleaner', 'manage_options',
            'cache_cleaner_settings',array(&$this, 'options_page'));
        if (current_user_can('manage_options'))
            add_filter('plugin_action_links', array(&$this, 'plugin_action_links'), 10, 2);
	}
	
	function options_page(){
        if (!empty($_POST['action']) && ($_POST['action']=='update') ){
			$this->options['period'] = $_POST['period'];
			$this->save_options();
			
			//Re-schedule the cache cleaner hook.
			wp_clear_scheduled_hook('clear_expired_cache');
			wp_schedule_event(time(), $this->options['period'], 'clear_expired_cache');
			
			echo '<div id="message" class="updated fade"><p><strong>Settings saved.</strong></p></div>';
		} 
        ?>
    <style type="text/css">
	/* Taken from http://sw-guide.de/wordpress/plugins/simple-trackback-validation/ */
	.wrap h3 { color: black; background-color: #e5f3ff; padding: 4px 8px; }
	</style>
    <div class="wrap">
    <h2>Cache Cleaner Options</h2>
    <form name="cache_cleaner_options" method="post" action="<?php echo $_SERVER['PHP_SELF']; ?>?page=cache_cleaner_settings">
    <input type='hidden' name='action' value='update' />
		<p class="submit"><input type="submit" name="Submit" value="Save Changes" /></p>
	<fieldset class="options">
		<h3>Cache Cleanup</h3>
				

		How often should expired files be deleted : 
		<br />
		<ul style='list-style-type: none;'>
			<li><label><input type='radio' name='period' value='hourly' 
			<?php if ($this->options['period']=='hourly') echo 'checked="checked"'; ?>/> Hourly</label></li>
			<li><label><input type='radio' name='period' value='daily' 
			<?php if ($this->options['period']=='daily') echo 'checked="checked"'; ?>/> Daily</label></li>
		</ul>
		
		The plugin will force WP Super Cache to delete expired cache files at least this often, 
		regardless of whether the number of requests set in 
		<a href='admin.php?page=wpsupercache'>Garbage Collection</a> is reached or not.
		
		
		
    </fieldset>
    
    <fieldset class="options">
		<h3>Info</h3>
		
		Last cache cleanup (by this plugin) : <?php
			if ($this->options['last_run']>0){
				echo date('Y-m-d H:i:s', $this->options['last_run']);
			} else {
				echo "Never";
			}
		 ?> 
		<br />
		<!--
		Last time the pugin was invoked : <?php
			if ($this->options['last_check']>0){
				echo date('Y-m-d H:i:s', $this->options['last_check']);
			} else {
				echo "Never";
			}
		 ?> 
		-->
    </fieldset>
    
		<p class="submit"><input type="submit" name="Submit" value="Save Changes" /></p>
	</form>
    </div>
        <?php
    }
    
    function plugin_action_links($links, $file) {
        if ($file == plugin_basename(__FILE__))
            $links[] = "<a href='options-general.php?page=cache_cleaner_settings'>" . __('Settings') . "</a>";
        return $links;
    }
	
} //class

$cache_cleaner = new CacheCleaner;

?>